#include <stdint.h>
#include <stdbool.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "lcd.h"
#include "uart.h"

#define FOSC	F_CPU
#define BAUD	19200 // 9600
#define UBRR	FOSC/16/BAUD-1



char buf[8];
unsigned char pos=0;

int d=255;


//define overflow counter
uint16_t ov_counter;
//define overall counts
uint32_t counts;

uint32_t last_length;
uint32_t lengths[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0};
unsigned char lpt=0;
uint32_t tmp;


ISR(TIMER1_OVF_vect){ // timer1 counter overflow
  ov_counter++;
}

ISR(TIMER1_CAPT_vect){ // timer1 input capture interrupt
	static unsigned char c=0;
	PORTC=c;c=~c;
	static unsigned int last=0;
	static unsigned int last_ov=0;
	unsigned int now=ICR1;
//	unsigned int sample=now-last;
/*
	if (ov_counter>last_ov+2){
	} else {
		//if ((now-last) < 10000){ // ignore short pulse
			TIFR1|=_BV(ICF1); // clear input capture flag
			return;
		//}
	}
*/
	tmp=ov_counter-last_ov;
	tmp=tmp<<16;
	last_length=tmp+(now-last);
	lengths[lpt++]=last_length;
	if (lpt==16) lpt=0;
	counts++;
	last_ov=ov_counter;

	TIFR1|=_BV(ICF1); // clear input capture flag
	last=now;
	return;
}

ISR(USART_RX_vect) {
	volatile unsigned char t=UDR0;
	switch(t){
		case 'a': UDR0=t; break;
		case '\r':
		case '\n':
			UDR0=t;
			transmit_1byte_USART0(t);
			break;
	}
}

char nibbles[]="0123456789abcdef";
void putint16hex(unsigned int i, unsigned char line, unsigned char column){
	unsigned char nibble;
	for (nibble=0;nibble<4;nibble++){
        SendCommand(0x80 + 0x40 * line + column - nibble);
	    SendData(nibbles[i&0xf]);
		i>>=4;
	}
}

int main(void)
{
//	DDRC  = 0xff; // portc = output

	DDRB=0; // portb = input
	PORTB=0x00; // disable pullups

    InitLCD();
	//init_USART0(UBRR);  // initialize USART0
	//setRXCIE_USART0();

	TIMSK1=
		_BV(ICIE1)| // enable input capture interrupt
		_BV(TOIE1)| // enable overflow interrupt
		0;
	TCCR1B=
		_BV(ICNC1)| // enable noise canceller
	//	_BV(ICES1)| // trigger on rising edge (without this: falling edge)
		_BV(CS10)|  // no prescaler, run at system clock
	//	_BV(CS11)|  // prescaler=8, run at system clock / 8
		0;



        ///SendLong(lnum, 2, 0);           // long
        //lnum+=27;
        
        //SendString("Hello World     ", 2, 10);    // ensure background cleared
        //SendInt(inum, 2, 10);           // int
        //inum++;

	//transmit_str_USART0("init done!\r\n");
	sei(); // enable interrupts
   
	uint32_t tmpi;
	float tmpf;
	unsigned char i;
	while(1) {
		// current timer/counter value
		//putint16hex(TCNT1,0,7);
		//putint16hex(ov_counter,0,3);

		// length of last pulse
		//putint16hex(last_length,0,15);
		//putint16hex(last_length>>16,0,11);

		// pulse count
		//putint16hex(counts,1,7);
		//putint16hex(counts>>16,1,3);

		
		// last_length is pulse length in 20Mhz clock pulses
		// meter sends 51200 pulses per kWh => 1 pulse = 1/51200 kWh

		tmpi=0;
		for (i=0;i<16;i++) tmpi+=lengths[i];
		tmpi/=16;

		tmpf=tmpi;//last_length;
		tmpf=tmpf/20/1000/1000; // pulse length in s
		
		tmpf=51200/tmpf/60/60*4;

		SendFloat(tmpf,1,1);
		SendString("W",1,13);

		tmpf=counts;
		tmpf/=51200;
		SendFloat(tmpf,2,1);
		SendString("kWh",2,13);


	}
    return 0;
}

